/***************************************************************************
 * LPNG - Solaris lp compatibility for LPRng
 *
 * Written by Garrett D'Amore
 *     garrett@yavin.org
 *
 * Copyright (C) 1997-1998 Garrett D'Amore.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ***************************************************************************
 * MODULE: cancel.c
 * PURPOSE: provides Solaris cancel compatiblity for LPRng
 **************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <pwd.h>
#include <time.h>
#include <netdb.h>
#include <pwd.h>
#include "tcp.h"
#include "strerror.h"
#include "version.h"

#define BUFSIZE 1024

char *whoami(void);

void usage()
{
    fprintf(stderr, "\
usage: cancel id ... printer ...\n\
  or   cancel -u id-list printer\n");
    exit(0);
}

char *chomp(char *string)
{
    char *probe;
    probe = strchr(string, '\n');
    if (probe) *probe = '\0';
    return string;
}

char *get_arg(int argc, char **argv, int *index)
{
    if (strlen(argv[*index]) > 2)
    {
	return argv[*index] + 2;
    }
    else if ((*index + 1 < argc) && (*argv[*index + 1] != '-'))
    {
	*index += 1;
	return argv[*index];
    }
    else
    {
	return 0;
    }
}

/*
 * Connect to the LPRng server running on the local host.
 */
FILE *lpd_connect()
{
    int		fd;
    FILE	*file;

    fd = tcpConnect("localhost", tcpService("printer"));
    if (fd < 0)
    {
	fprintf(stderr, "\
UX:cancel: ERROR: Can't connect to LPRng server (%s).\n\
          TO FIX: Contact your system administrator.\n",
		tcpStrError());
	return NULL;
    }
	
    file = fdopen(fd, "w+");
    if (!file)
    {
	fprintf(stderr, "\
UX:cancel: ERROR: Can't fdopen the connection (%s).\n\
          TO FIX: Contact your system administrator.\n",
		strerror(errno));
	close(fd);
	return NULL;
    }
    return file;
}

void tr(char *string, char a, char b)
{
    char *probe;
    for (probe = string; *probe; probe++) { if (*probe = a) *probe = b; }
}

int dequeue(const char *queue, char *list)
{
    char	buffer [ BUFSIZE ];
    int		dequeued_found = 0;
    const	char *id;
    FILE	*lprng;

    lprng = lpd_connect();
    if (!lprng) return -1;

    fprintf(lprng, "\05%s %s %s\n", queue, whoami(), list);
    fflush(lprng);
    rewind(lprng);

    while (fgets(buffer, sizeof(buffer) - 1, lprng) != NULL)
    {
	/* check for LPRng format dequeue messages */
	if (!strncmp(buffer, "  dequeued", strlen("  dequeued")))
	{
	    id = strchr(buffer, '+');
	    id = id ? id + 1 : NULL;
	    dequeued_found = 1;
	}
	else if (strstr(buffer, "dequeued") && strstr(buffer, ": cf"))
	{
	    const char *id = strstr(buffer, ": cf");
	    id = id ? id + 5  : NULL;
	    dequeued_found = 1;
	}
	else
	{
	    continue;
	}

	if (id)
	{
	    printf("request \"%s-%03d\" cancelled\n", queue, atoi(id));
	}
	else
	{
	    printf("request \"%s-???\" cancelled\n", queue);
	}
    }
    fclose(lprng);
    return dequeued_found ? 0 : 1;
}

char *whoami(void)
{
    int uid;
    struct passwd *pwd;
    
    uid = getuid();
    pwd = getpwuid(uid);
    if (!pwd)
    {
	return "nobody";
    }
    return pwd->pw_name;
}

int main(int argc, char **argv)
{
    int i;
    int optchar;
    int doing_printers = 0;
    char *userlist = NULL;

    extern int opterr;
    extern char *optarg;
    extern int optind;
    extern int optopt;

    opterr = 0;
    while ((optchar = getopt(argc, argv, "u:V")) != EOF)
    {
	switch (optchar)
	{
	    case 'V' :
		puts(CONFIGURATION);
		exit(0);

	    case 'u' :
		userlist = optarg;
		doing_printers = 1;
		break;

	    case '?':
	    	fprintf(stderr, optopt == 'u' ?
		        "UX:cancel: ERROR: The option -%c requires an argument.\n" :
		        "UX:cancel: ERROR: Unrecognized option \"-%c\".\n",
			optopt);
		exit(1);
		break;
	}
    }

    for (i = optind; (userlist == NULL) && (!doing_printers) && (i < argc); i++)
    {
	/* the problem is deciding what is a queue name and what is a
	 * job id.  nasty problem -- which is why SYSV doesn't allow
	 * dashes in queue names.  puke!
	 */
	if (strchr(argv[i], '-'))
	{
	    char *queue;
	    char *jobid;
	
	    queue = strtok(argv[i], "-");
	    jobid = strtok(NULL, "");
	    if (!jobid) jobid = "";

	    /* dequeue the job */
	    dequeue(queue, jobid);
	}
	else
	{
	    doing_printers = 1;
	    break;
	}
    }
    
    if (optind >= argc)
    {
	/* cancel wasn't given any printers or request-ids, but maybe we
	 * have a userlist to use instead */
	if (userlist)
	{
	    dequeue("all", userlist);
	}
	else
	{
	    fprintf(stderr, "\
UX:cancel: ERROR: Nothing to do.\n\
          TO FIX: You need to give a request ID or printer\n\
                  name, or use the -u option to specify a user.\n");
	    exit(1);
	}
    }

    /* at this point, everything left is a printer.  we either try to remove
     * all jobs associated with the named userlist, or just the active job */
    for (/* i = i */; i < argc; i++)
    {
	dequeue(argv[i], userlist ? userlist : "");
    }

    return 0;
}
