/***************************************************************************
 * LPNG - Solaris lp compatibility for LPRng
 *
 * Written by Garrett D'Amore
 *     garrett@yavin.org
 *
 * Copyright (C) 1997-1998 Garrett D'Amore.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 ***************************************************************************
 * MODULE: lpstat.c
 * PURPOSE: provides Solaris lpstat compatiblity for LPRng
 **************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <pwd.h>
#include <time.h>
#include <netdb.h>
#include <pwd.h>
#include "tcp.h"
#include "strerror.h"
#include "version.h"

#define BUFSIZE 1024

/* these come from LPRng's src/LPD/lpd_status.c */
#define RANKW	7
#define OWNERW	30
#define CLASSW	1
#define JOBW	4
#define FILEW	18
#define SIZEW	5
#define TIMEW	8

typedef struct
{
    char queue  [ BUFSIZE ];
    char device [ BUFSIZE ];
    char server [ BUFSIZE ];
    char remote [ BUFSIZE ];
    char desc   [ BUFSIZE ];
} QUEUE;

typedef struct
{
    char queue [ BUFSIZE ];
    int	 active_job;
    int  printing_enabled;
    int  spooling_enabled;
    int  unspooler_active;
    int  not_configured;
} STATUS;

void usage()
{
    fprintf(stderr, "\
usage:
    lpstat [options] [request-ids]\n\
	[-a [printers]]		(show acceptance status)\n\
	[-c [classes]]		(ignored, for compatibility only)\n\
	[-d]			(show default destination)\n\
	[-D]			(ignored, for compatibility only)\n\
	[-f [forms]]		(ignored, for compatibility only)\n\
	[-l]			(ignored, for compatibility only)\n\
	[-o [printers,req-ids]]	(show status of requests)\n\
	[-p [printers]]		(show available printers)\n\
	[-P]			(ignored, for compatibility only)\n\
	[-R]			(show rank in queue)\n\
	[-r]			(show status of LPRng daemon)\n\
	[-s]			(show summary status)\n\
	[-S [charsets]]		(ignored, for compatibility only)\n\
	[-t]			(show status of everything)\n\
	[-u [users]]		(show status of user requests)\n\
	[-v [printers]]		(show printer destinations)\n\
	[-H]			(ignored, for compatibility only)\n\
		(\"all\" allowed with -a,-o,-p,-u,-v options)\n");
    exit(0);
}

char *chomp(char *string)
{
    char *probe;
    probe = strchr(string, '\n');
    if (probe) *probe = '\0';
    return string;
}

char *get_arg(int argc, char **argv, int *index)
{
    if (strlen(argv[*index]) > 2)
    {
	return argv[*index] + 2;
    }
    else if ((*index + 1 < argc) && (*argv[*index + 1] != '-'))
    {
	*index += 1;
	return argv[*index];
    }
    else
    {
	return "all";
    }
}

/*
 * Connect to the LPRng server running on the local host.
 */
FILE *lpd_connect()
{
    int		fd;
    FILE	*file;

    fd = tcpConnect("localhost", tcpService("printer"));
    if (fd < 0)
    {
	fprintf(stderr, "\
UX:lpstat: ERROR: Can't connect to LPRng server (%s).\n\
          TO FIX: Contact your system administrator.\n",
		tcpStrError());
	return NULL;
    }
	
    file = fdopen(fd, "w+");
    if (!file)
    {
	fprintf(stderr, "\
UX: lpstat: ERROR: Can't fdopen the connection (%s).\n\
           TO FIX: Contact your system administrator.\n",
		strerror(errno));
	close(fd);
	return NULL;
    }
    return file;
}

void no_printer(const char *printer)
{
	fprintf(stderr, "\
UX: lpstat: ERROR: Printer \"%s\" does not exist.\n\
           TO FIX: Use the \"lpstat -p all\" to list\n\
                   all known printers.\n", printer);
	return;
}

/*
 * Check to see if LPRng's lpd is running, and report back appropriately.
 */
int lpd_is_running()
{
    int fd = tcpConnect("localhost", tcpService("printer"));
    if (fd < 1)
    {
	printf("scheduler is not running\n");
    }
    else
    {
	printf("scheduler is running\n");
	close(fd);
    }
    return 0;
}

int get_status(FILE *file, STATUS *status)
{
    char	buffer [ BUFSIZE ];
    char	*probe;
    int		peek;

    memset(status, 0, sizeof(STATUS));

    /* look for start of new queue entry */
    while (fgets(buffer, BUFSIZE - 1, file) != NULL)
    {
	/* ensure null termination and strip newline */
	buffer [ BUFSIZE - 1 ] = '\0';
	buffer [ strlen(buffer) - 1 ] = '\0';
	if (strstr(buffer, "Printer: ") == buffer) break;
	
    }

    strcpy(status->queue, "unknown");
    status->active_job = 0;
    status->unspooler_active = 0;
    status->printing_enabled = 1;
    status->spooling_enabled = 1;
    
    if (strstr(buffer, "not in printcap"))
    {
	status->not_configured = 1;
	status->printing_enabled = 0;
	status->spooling_enabled = 0;
    }
    else
    {
	if (strstr(buffer, "printing disabled")) status->printing_enabled = 0;
	if (strstr(buffer, "spooling disabled")) status->spooling_enabled = 0;
    }
    strcpy(status->queue, buffer + strlen("Printer: "));


    /* truncate the queue at the field name */
    probe = strchr(status->queue, '@');
    if (probe) *probe = '\0';

    /* now read the data for each queue */
    while (1)
    {
	peek = getc(file);

	if (peek == EOF) return 0;
	ungetc(peek, file);

	/* if this is the start of a new queue, skip ahead 
	 * this heuristic will get confused if the user name starts with
	 * a capital 'P'.  (unlikely on modern UNIX systems.)
	 */
	if (peek == 'P') return 1;

	if (fgets(buffer, BUFSIZE - 1, file) == NULL) return 0;

	/* is there an unspooler active? -- LPRng */
	if (!strncmp(buffer, " Unspooler:", strlen(" Unspooler:")))
	{
	    status->unspooler_active = 1;
	}

	/* is there an unspooler active? -- BSD */
	else if (strstr(buffer, "ready and printing"))
	{
	    status->unspooler_active = 1;
	}

	/* find the active job -- LPRng */
	else if (!strncmp(buffer, "active", strlen("active")))
	{
	    char *probe;

	    probe = strchr(buffer, '+');
	    if (probe)
	    {
		status->unspooler_active = 1;
		status->active_job = atoi(++probe);
	    }
	}
	
	/* find the active job -- BSD */
	else if (strstr(buffer,  ": active"))
	{
	    char *probe;

	    probe = strstr(buffer, "[job ");
	    if (probe)
	    {
		char jobid [4];

		strncpy(jobid, buffer, 3);
		jobid [3] = '\0';
		status->unspooler_active = 1;
		status->active_job = atoi(jobid);
	    }
	}
    }
}

/*
 * Get the configuration information for a queue.
 */
int get_queue(FILE *file, QUEUE *queue)
{
    char buffer [ BUFSIZE ] = "";
    int  peek;

    /* clear the structure */
    memset(queue, 0, sizeof(QUEUE));

    /* look for start of new printer config */
    while (fgets(buffer, BUFSIZE - 1, file) != NULL)
    {
	/* ensure null termination and strip newline */
	buffer [ BUFSIZE - 1 ] = '\0';
	chomp(buffer);
	if (!isspace(buffer[0])) break;
    }

    strncpy(queue->queue, buffer, BUFSIZE - 1);
    if (!queue->queue[0]) return -1;

    while (1)
    {
	/* peek a head to make sure next line isn't a definition start */
	peek = getc(file);

	/* no more data! */
	if (peek == EOF) return 0;

	/* oops, too far, go back and finish this scan sequence */
	if (!isspace(peek))
	{
	    ungetc(peek, file);
	    return 1;
	}

	if (fgets(buffer, BUFSIZE - 1, file) == NULL) return 0;
	buffer [ BUFSIZE - 1 ] = '\0';
	buffer [ strlen(buffer) - 1 ] = '\0';

	/* now look for the tags we know about */
	if (!strncmp(buffer, ":lp=", 4))
	{
	    strncpy(queue->device, buffer + 4, BUFSIZE - 1);
	}
	else if (!strncmp(buffer, ":rm=", 4))
	{
	    strncpy(queue->server, buffer + 4, BUFSIZE - 1);
	}
	else if (!strncmp(buffer, ":rp=", 4))
	{
	    strncpy(queue->remote, buffer + 4, BUFSIZE - 1);
	}
	else if (!strncmp(buffer, ":cm=", 4))
	{
	    strncpy(queue->desc, buffer + 4, BUFSIZE - 1);
	}
	/* we skip any other junk we don't know about */
    }
}

int lpd_status(char *list,
	       char *whence,
    	       int showrank)
{
    char *item;
    char *user;
    char *system;
    char *saved;
    FILE *lprng;
    char *job_owner;
    char *job_host;
    char *job_time;
    int  job_size;
    int	 job_number;
    int	 job_rank;
    int  state = 0; 	/* 1 = LPRng, 2 = Traditional BSD */
    char buffer [ BUFSIZE ];
    char queue  [ BUFSIZE ];
    char jobn	[ BUFSIZE ];
    int  erred = 0;

    if (whence && strchr(whence, '!'))
    {
	system = strtok(whence, "!");
	user = strtok(whence, "");
    }
    else if (whence)
    {
	system = "all";
	user = whence;
    }
    else
    {
	system = "all";
	user = "";
    }

    
    item = strtok(list, ", ");
    while (item != NULL)
    {
	saved = strtok(NULL, ", ");
	lprng = lpd_connect();
	if (!lprng) continue;

	fprintf(lprng, "\04%s %s %s\n",
		isdigit(item[0]) ? "all" : item,
		user ? user : "",
		isdigit(item[0]) ? item : "");

	/* flush the output and prepare to read responses */
	fflush(lprng);
	rewind(lprng);

	strcpy(queue, "");
	while (fgets(buffer, BUFSIZE - 1, lprng) != NULL)
	{
	    chomp(buffer);
	    if (!strncmp(buffer, "Printer: ", strlen("Printer: ")))
	    {
		char *probe;
		if (strstr(buffer, "not in printcap"))
		{
		    erred = 1;
		    probe = buffer + strlen("Printer: ");
		    strtok(probe, "@ \t");
		    strncpy(queue, probe, BUFSIZE - 1);
		    no_printer(queue);
		}
		else
		{
		    probe = buffer + strlen("Printer: ");
		    strtok(probe, "@ \t");
		    strncpy(queue, probe, BUFSIZE - 1);
		}
		state = 0;
		continue;
	    }
	    
	    /* we don't know what kind of LPQ output to expect yet */
	    if (state == 0)
	    {
		if (strstr(buffer, "[job"))
		{
		    state = 2;
		}
		else if (!strncmp(buffer, " Rank", 5))
		{
		    state = 1;
		    continue;
		}
	    }

	    /* this is LPRng style LPQ output */
	    if (state == 1)
	    {
		char *token;
		char *probe;
		char buf [ BUFSIZE ];
		time_t now;
		struct tm *ltime;

		now = time(NULL);
		ltime = localtime(&now);
		strftime(buf, BUFSIZE - 1, "%b %d ", ltime);

		/* make sure that our buffer is wide enough */
		if (strlen(buffer) <
		    (RANKW +1+ OWNERW +1+ CLASSW +1+ JOBW +1+ FILEW))
		    continue;

		/* rank */
		buffer [RANKW] = '\0';
		token = buffer;
		strtok(token, " \t");
		
		if (!strcmp(token, "active"))
		{
		    job_rank = 1;
		}
		else
		{
		    job_rank = atoi(token);
		}
		/* ensure job rank is something we recognize */
		if (!job_rank) continue;

		/* job owner and host */
		token += (RANKW + 1);
		token [OWNERW] = '\0';
		job_owner = strtok(token, "@");
		if (!job_owner) continue;
		job_host = strtok(NULL, "+");
		if (!job_host) continue;

		/* class */
		token += (OWNERW + 1);

		/* job number */
		token += (CLASSW + 1);
		probe = strtok(token, " ");
		if (!probe) continue;
		
		job_number = atoi(probe);

		/* files */
		token += (JOBW + 1);

		/* size & time */
		token += (FILEW + 1);
		probe = strtok(token, " ");
		if (!probe) continue;
		job_size = atoi(probe);

		probe = strtok(NULL, "\r\n");
		if (!probe) continue;
		job_time = probe;
		probe = strrchr(job_time, ':');
		/* remove seconds */
		if (probe) *probe = '\0';
		strncat(buf, job_time, BUFSIZE - 1);
		job_time = buf;
	    }

	    /* BSD traditional LPD output */
	    else if (state == 2)
	    {
		char *probe;
		static char buf2 [ BUFSIZE ];

		strcpy(buf2, buffer );

		probe = strstr(buf2, "[job ");
		if (!probe) continue;

		*probe = '\0';
		probe += strlen("[job ");

		job_number = atoi(probe);
		job_host = probe + strspn(probe, "0123456789 ");
		strtok(job_host, "\t\r\n]");
		
		job_owner = strtok(buf2, ": \t");
		probe = strtok(NULL, "");
		job_rank = atoi(probe ? probe : "0");
		
		state = 3;
		continue;
	    }

	    /* 2nd line of BSD LPD output */
	    else if (state == 3)
	    {
		char buf [ BUFSIZE ];
		time_t now;
		struct tm *ltime;

		now = time(NULL);
		ltime = localtime(&now);
		
		if (buffer[0] != '\t')
		{
		    state = 2;	/* erroneous output resets state */
		    continue;
		}
		
		if (strlen(buffer) < 33)
		{	
		    state = 2;
		    continue;
		}
		
		job_size = atoi(buffer + 33);
		strftime(buf, BUFSIZE - 1, "%b %d %R", ltime);
		job_time = strtok(buf, "\n");
	    }
	    
	    if (!state) continue;

	    /* system names need to be canonicalized to get domain name */
	    if (system && strcmp(system, "all"))
	    {
		char buf1 [ BUFSIZE ];
		char buf2 [ BUFSIZE ];
		struct hostent *he;
		he = gethostbyname(system);
		if (he)
		{
		    /* canonicalize the name */
		    strncpy(buf1, he->h_name, BUFSIZE - 1);
		    buf1 [ BUFSIZE - 1] = '\0';
		}
		else
		{
		    strcpy(buf1, system);
		}
		
		he = gethostbyname(job_host);
		if (he)
		{
		    /* canonlicalize the name */
		    strncpy(buf2, he->h_name, BUFSIZE - 1);
		    buf2 [ BUFSIZE - 1] = '\0';
		}
		else
		{
		    strcpy(buf2, job_host);
		}
		if (strcmp(buf1, buf2))
		{
		    continue;
		}
	    }

	    if (showrank) printf("%3d ", job_rank);
	    
	    printf("%s-%03d%*s%s!%s%8d %s\n",
		   queue,
		   job_number,
		   24 - strlen(queue) - 3 - 1,
		   "",
		   job_host,
		   job_owner,
		   job_size,
		   job_time);
	}
	fclose(lprng);
	item = saved ? strtok(saved, " ,") : NULL;
    }
    return erred;
}

int show_device(QUEUE queue)
{
    if (!queue.queue[0]) return 1;
    if (queue.server[0])
    {
	printf("system for %s: %s", queue.queue, queue.server);
	if (queue.remote[0])
	{
	    printf(" (as printer %s)", queue.remote);
	}
	printf("\n");
    }
    else
    {
	printf("device for %s: %s\n", queue.queue, queue.device);
    }
    return 0;
}

char *whoami(void)
{
    int uid;
    struct passwd *pwd;
    
    uid = getuid();
    pwd = getpwuid(uid);
    if (!pwd)
    {
	return "nobody";
    }
    return pwd->pw_name;
}

/* TODO: this should really parse the config files to get the real
 *	 system default destination, but I'm too lazy to do it right
 *	 now.
 */
int lpd_default(void)
{
    char buffer [ BUFSIZE ];
    FILE *lprng;
    char *printer = 0;

    lprng = lpd_connect();
    if (!lprng) exit(1);
    fprintf(lprng, "\006all %s defaultq\n", whoami());
    fflush(lprng);
    rewind(lprng);

    if (fgets(buffer, BUFSIZE - 1, lprng) == NULL) strcpy(buffer, "");
    fclose(lprng);

    buffer [ BUFSIZE - 1 ] = '\0';
    printer = strtok(buffer, " \t\r\n");
    if (printer)
    {
	printf("system default destination: %s\n", printer);
    }
    else
    {
	printf("no system default destination\n");
    }
    return 0;
}

/* TODO: it would be better if we could get this information from the
 *       local server directly, but that requires simple protocol changes
 *	 to prevent the server from contacting the remote host.  For now
 *	 we just fake the date by using the current time (bletch).
 */
int lpd_accepting(char *list)
{
    char *printer;
    char *saved;
    FILE *lprng;
    time_t now;
    int erred = 0;

    printer = strtok(list, ", ");
    while (printer != NULL)
    {
	saved = strtok(NULL, "");
	
	lprng = lpd_connect();
	if (!lprng) exit(1);
	fprintf(lprng, "\006%s %s status\n", printer, whoami());
	fflush(lprng);
	rewind(lprng);
	now = time(NULL);
	
	while (1)
	{
	    char buffer [ BUFSIZE ];
	    char *printer;
	    char *spooling;
	    char *printing;

	    if (fgets(buffer, BUFSIZE - 1, lprng) == NULL) break;

	    /* skip header */
	    if (isspace(buffer[0])) continue;

	    chomp(buffer);
	    printer = strtok(buffer, " \t\r\n");
	    if (strstr(buffer + strlen(printer) + 1, "not in printcap"))
	    {
	        strtok(printer, "@");
		no_printer(printer);
		erred = 1;
		continue;
	    }
	    printing = strtok(NULL, " \t\r\n");
	    spooling = strtok(NULL, " \t\r\n");
	    if (!printer) continue;
	    if (!printing) continue;
	    if (!spooling) continue;
	    strtok(printer, "@");

	    if (!strcmp(spooling, "enabled"))
	    {
		printf("%s accepting requests since %s", printer, ctime(&now));
	    }
	    else if (!strcmp(spooling, "disabled"))
	    {
		printf("%s not accepting requests since %s", printer,
		       ctime(&now));
		printf("\tunknown reason\n");
	    }
	}
	fclose(lprng);
	printer = saved ? strtok(saved, ", ") : NULL;
    }
    return erred;
}

/*
 * Show the output of lpstat -p.  This uses the LPQ VERBOSE output and parses
 * it.  Thanks to the change in the format used by LPRng (which I don't like),
 * the parsing is tricky.  Also, because LPRng's LPD forwards LPQ requests to
 * remote print servers this can take a little while to respond at sites with
 * lots of print servers or when print servers are down.  Applications should
 * not use this interface to determine available queues -- lpstat -a and
 * lpstat -v are preferred.
 */
int lpd_printers(char *list)
{
    char	*printer;
    char	*saved;
    FILE	*lprng;
    STATUS	status;
    time_t	now;
    char	nowstr [ BUFSIZE ];
    int		result;
    int		erred = 0;

    printer = strtok(list, ", ");
    while (printer)
    {
	saved = strtok(NULL, "");

	now = time (NULL);
	strcpy(nowstr, ctime(&now));
	strtok(nowstr, "\n");
	
	lprng = lpd_connect();
	if (lprng == NULL) exit(1);
	
	fprintf(lprng, "\004%s\n", printer);
	fflush(lprng);
	rewind(lprng);
		
	do
	{
	    result = get_status(lprng, &status);
	    if (result >= 0)
	    {
		if (status.not_configured)
		{
		    no_printer(status.queue);
		    erred = 1;
		    continue;
		}
		else if (status.unspooler_active)
		{
		    printf("printer %s now printing %s-%03d. ",
			   status.queue, status.queue, status.active_job);
		}
		else
		{
		    printf("printer %s is idle. ", status.queue);
		}
		printf("%sabled since %s. %savailable.\n",
		       status.printing_enabled ? "en" : "dis",
		       nowstr,
		       status.spooling_enabled ? "" : "un");
	    }
	} while (result > 0);
	fclose(lprng);
	printer = strtok(saved, ", ");
    }
}

/*
 * Show the output of lpstat -v.
 */
int lpd_device(char *list)
{
    char *printer;
    int result = 0;
    QUEUE queue;
    FILE *lprng;
    int erred = 0;
    
    printer = strtok(list, ", ");
    while (printer)
    {
	lprng = lpd_connect();
	if (lprng == NULL) exit(1);
	
	fprintf(lprng, "\006%s %s printcap\n",	printer, whoami());
	fflush(lprng);
	rewind(lprng);

	do
	{
	    result = get_queue(lprng, &queue);
	    if (result < 0)
	    {
		if (strcmp(printer, "all"))
		{
		    erred = 1;
		    no_printer(printer);
		}
	    }
	    if (result >= 0) show_device(queue);
	}
	while (result > 0);

	fclose(lprng);
	printer = strtok(NULL, ", ");
    }
    return erred;
}

int main(int argc, char **argv)
{
    int optchar;
    char *optarg;
    int i;
    int show_rank = 0;
    int doing_args = 1;
    int erred = 0;

    for (i = 1; i < argc; i++)
    {
	if (doing_args && argv[i][0] == '-')
	{
	    optarg = "all";
	    optchar = argv[i][1];
	    switch (optchar)
	    {
	    case 'l':	/* long description, ignored */
	    case 'D':	/* description, ignored */
	    case 'P':	/* paper types, ignored */
	    case 'S':	/* character sets, ignored */
	    case 'H':	/* hardcoded terminal names, ignored */
		break;
	    
	    case 'R':	/* show rank */
		show_rank = 1;
		break;
		
	    case 's':	/* status summary */
		erred = lpd_is_running() ? 1 : erred;
		erred = lpd_default() ? 1 : erred;
		erred = lpd_device("all") ? 1 : erred;
		break;

	    case 't':
		erred = lpd_is_running() ? 1 : erred;
		erred = lpd_default() ? 1 : erred;
		erred = lpd_device("all") ? 1 : erred;
		erred = lpd_accepting("all") ? 1 : erred;
		erred = lpd_printers("all") ? 1 : erred;
		break;
		
	    case 'r':	/* scheduler running? */
		erred = lpd_is_running() ? 1 : erred;
		break;

	    case 'd':	/* default destination */
		erred = lpd_default() ? 1 : erred;
		break;

	    case 'f':	/* form names, ignored */
	    case 'c':	/* printer classes, ignored */
		optarg = get_arg(argc, argv, &i);
		break;
		
	    case 'v':	/* device configuration */
		optarg = get_arg(argc, argv, &i);
		erred = lpd_device(optarg) ? 1 : erred;
		break;
	    
	    case 'a':	/* acceptance status */
		optarg = get_arg(argc, argv, &i);
		erred = lpd_accepting(optarg) ? 1 : erred;
		break;

	    case 'o':	/* output requests */
		optarg = get_arg(argc, argv, &i);
		erred = lpd_status(optarg, 0, show_rank) ? 1 : erred;
		break;
		
	    case 'u':	/* users and systems */
		optarg = get_arg(argc, argv, &i);
		erred = lpd_status("all", optarg, show_rank) ? 1 : erred;
		break;

	    case 'p':	/* printer statuses */
		optarg = get_arg(argc, argv, &i);
		erred = lpd_printers(optarg) ? 1 : erred;
		break;

	    case 'V':	/* version information */
		puts(CONFIGURATION);
		break;

	    case '?':
		usage();
		break;

	    default:
		fprintf(stderr, "\
UX: lpstat: ERROR: Unrecognized option \"-%c\".\n\
                   Enter command with -? option for proper usage.\n",
			optchar);
		exit(1);
	    }
	}
	else
	{
	    doing_args = 0;
	    erred = lpd_status(argv[i], 0, show_rank) ? 1 : erred;
	}
	
    }
    
    /* no arguments is the same as saying lpstat on getuid */
    if (argc < 2)
    {
	char *lpdest;
	lpdest = getenv("LPDEST");
	if (!lpdest) lpdest = getenv("PRINTER");
	if (!lpdest) lpdest = "lp";
	erred = lpd_status(lpdest, "all", show_rank) ? 1 : erred;
    }

    return erred;
}
